package sun.security.validator;

import java.util.Arrays;
import java.util.List;
import java.util.Collections;
import java.util.Iterator;
import java.util.Set;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Collection;

class EndEntityChecker
{
    private static final String OID_EXTENDED_KEY_USAGE = "2.5.29.37";
    private static final String OID_EKU_TLS_SERVER = "1.3.6.1.5.5.7.3.1";
    private static final String OID_EKU_TLS_CLIENT = "1.3.6.1.5.5.7.3.2";
    private static final String OID_EKU_CODE_SIGNING = "1.3.6.1.5.5.7.3.3";
    private static final String OID_EKU_TIME_STAMPING = "1.3.6.1.5.5.7.3.8";
    private static final String OID_EKU_ANY_USAGE = "2.5.29.37.0";
    private static final String OID_EKU_NS_SGC = "2.16.840.1.113730.4.1";
    private static final String OID_EKU_MS_SGC = "1.3.6.1.4.1.311.10.3.3";
    private static final String OID_SUBJECT_ALT_NAME = "2.5.29.17";
    private static final String NSCT_SSL_CLIENT = "ssl_client";
    private static final String NSCT_SSL_SERVER = "ssl_server";
    private static final String NSCT_CODE_SIGNING = "object_signing";
    private static final int KU_SIGNATURE = 0;
    private static final int KU_KEY_ENCIPHERMENT = 2;
    private static final int KU_KEY_AGREEMENT = 4;
    private static final Collection<String> KU_SERVER_SIGNATURE;
    private static final Collection<String> KU_SERVER_ENCRYPTION;
    private static final Collection<String> KU_SERVER_KEY_AGREEMENT;
    private final String variant;
    private final String type;
    
    private EndEntityChecker(String type, String variant) {
        this.type = type;
        this.variant = variant;
    }
    
    static EndEntityChecker getInstance(String s, String s2) {
        return new EndEntityChecker(s, s2);
    }
    
    void check(X509Certificate[] array, Object o, boolean b) throws CertificateException {
        if (this.variant.equals("generic")) {
            return;
        }
        Set<String> criticalExtensions = this.getCriticalExtensions(array[0]);
        if (this.variant.equals("tls server")) {
            this.checkTLSServer(array[0], (String)o, criticalExtensions);
        }
        else if (this.variant.equals("tls client")) {
            this.checkTLSClient(array[0], criticalExtensions);
        }
        else if (this.variant.equals("code signing")) {
            this.checkCodeSigning(array[0], criticalExtensions);
        }
        else if (this.variant.equals("jce signing")) {
            this.checkCodeSigning(array[0], criticalExtensions);
        }
        else if (this.variant.equals("plugin code signing")) {
            this.checkCodeSigning(array[0], criticalExtensions);
        }
        else if (this.variant.equals("tsa server")) {
            this.checkTSAServer(array[0], criticalExtensions);
        }
        else {
            throw new CertificateException("Unknown variant: " + this.variant);
        }
        if (b) {
            this.checkRemainingExtensions(criticalExtensions);
        }
        Iterator<Object> iterator = CADistrustPolicy.POLICIES.iterator();
        while (iterator.hasNext()) {
            ((CADistrustPolicy)iterator.next()).checkDistrust(this.variant, array);
        }
        return;
    }
    
    private Set<String> getCriticalExtensions(X509Certificate x509Certificate) {
        Set<String> set = x509Certificate.getCriticalExtensionOIDs();
        if (set == null) {
            set = Collections.emptySet();
            return set;
        }
        return set;
    }
    
    private void checkRemainingExtensions(Set<String> obj) throws CertificateException {
        obj.remove("2.5.29.19");
        obj.remove("2.5.29.17");
        if (!obj.isEmpty()) {
            throw new CertificateException("Certificate contains unsupported critical extensions: " + obj);
        }
        return;
    }
    
    private boolean checkEKU(X509Certificate x509Certificate, Set<String> set, String s) throws CertificateException {
        List<String> extendedKeyUsage = x509Certificate.getExtendedKeyUsage();
        return extendedKeyUsage == null || extendedKeyUsage.contains(s) || extendedKeyUsage.contains("2.5.29.37.0");
    }
    
    private boolean checkKeyUsage(X509Certificate x509Certificate, int n) throws CertificateException {
        boolean[] keyUsage = x509Certificate.getKeyUsage();
        return keyUsage == null || (keyUsage.length > n && keyUsage[n]);
    }
    
    private void checkTLSClient(X509Certificate x509Certificate, Set<String> set) throws CertificateException {
        if (!this.checkKeyUsage(x509Certificate, 0)) {
            throw new ValidatorException("KeyUsage does not allow digital signatures", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        if (!this.checkEKU(x509Certificate, set, "1.3.6.1.5.5.7.3.2")) {
            throw new ValidatorException("Extended key usage does not permit use for TLS client authentication", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        if (!SimpleValidator.getNetscapeCertTypeBit(x509Certificate, "ssl_client")) {
            throw new ValidatorException("Netscape cert type does not permit use for SSL client", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        set.remove("2.5.29.15");
        set.remove("2.5.29.37");
        set.remove("2.16.840.1.113730.1.1");
        return;
    }
    
    private void checkTLSServer(X509Certificate x509Certificate, String str, Set<String> set) throws CertificateException {
        if (EndEntityChecker.KU_SERVER_ENCRYPTION.contains(str)) {
            if (!this.checkKeyUsage(x509Certificate, 2)) {
                throw new ValidatorException("KeyUsage does not allow key encipherment", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
            }
        }
        else if (EndEntityChecker.KU_SERVER_SIGNATURE.contains(str)) {
            if (!this.checkKeyUsage(x509Certificate, 0)) {
                throw new ValidatorException("KeyUsage does not allow digital signatures", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
            }
        }
        else if (EndEntityChecker.KU_SERVER_KEY_AGREEMENT.contains(str)) {
            if (!this.checkKeyUsage(x509Certificate, 4)) {
                throw new ValidatorException("KeyUsage does not allow key agreement", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
            }
        }
        else {
            throw new CertificateException("Unknown authType: " + str);
        }
        if (!this.checkEKU(x509Certificate, set, "1.3.6.1.5.5.7.3.1") && !this.checkEKU(x509Certificate, set, "1.3.6.1.4.1.311.10.3.3") && !this.checkEKU(x509Certificate, set, "2.16.840.1.113730.4.1")) {
            throw new ValidatorException("Extended key usage does not permit use for TLS server authentication", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        if (!SimpleValidator.getNetscapeCertTypeBit(x509Certificate, "ssl_server")) {
            throw new ValidatorException("Netscape cert type does not permit use for SSL server", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        set.remove("2.5.29.15");
        set.remove("2.5.29.37");
        set.remove("2.16.840.1.113730.1.1");
        return;
    }
    
    private void checkCodeSigning(X509Certificate x509Certificate, Set<String> set) throws CertificateException {
        if (!this.checkKeyUsage(x509Certificate, 0)) {
            throw new ValidatorException("KeyUsage does not allow digital signatures", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        if (!this.checkEKU(x509Certificate, set, "1.3.6.1.5.5.7.3.3")) {
            throw new ValidatorException("Extended key usage does not permit use for code signing", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        if (!this.variant.equals("jce signing")) {
            if (!SimpleValidator.getNetscapeCertTypeBit(x509Certificate, "object_signing")) {
                throw new ValidatorException("Netscape cert type does not permit use for code signing", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
            }
            set.remove("2.16.840.1.113730.1.1");
        }
        set.remove("2.5.29.15");
        set.remove("2.5.29.37");
        return;
    }
    
    private void checkTSAServer(X509Certificate x509Certificate, Set<String> set) throws CertificateException {
        if (!this.checkKeyUsage(x509Certificate, 0)) {
            throw new ValidatorException("KeyUsage does not allow digital signatures", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        if (x509Certificate.getExtendedKeyUsage() == null) {
            throw new ValidatorException("Certificate does not contain an extended key usage extension required for a TSA server", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        if (!this.checkEKU(x509Certificate, set, "1.3.6.1.5.5.7.3.8")) {
            throw new ValidatorException("Extended key usage does not permit use for TSA server", ValidatorException.T_EE_EXTENSIONS, x509Certificate);
        }
        set.remove("2.5.29.15");
        set.remove("2.5.29.37");
        return;
    }
    
    static {
        KU_SERVER_SIGNATURE = Arrays.asList(new String[] { "DHE_DSS", "DHE_RSA", "ECDHE_ECDSA", "ECDHE_RSA", "RSA_EXPORT", "UNKNOWN" });
        KU_SERVER_ENCRYPTION = Arrays.asList(new String[] { "RSA" });
        KU_SERVER_KEY_AGREEMENT = Arrays.asList(new String[] { "DH_DSS", "DH_RSA", "ECDH_ECDSA", "ECDH_RSA" });
    }
}
